<?php
if (preg_match('/\.(?:css|png|jpg|jpeg|gif)$/', $_SERVER["REQUEST_URI"])) {
    return false;    // serve the requested resource as-is.
}

include 'config/config.php';

foreach ($config['core_classes'] as $classname){
    require "core/$classname.php";
}

function get_path_elements() {
    if (!isset($_SERVER['PATH_INFO'])) return null;
    $path_info = $_SERVER['PATH_INFO'];
    $path_elements = explode('/', $path_info);
    array_shift($path_elements);
    return $path_elements;
}
function path_contains_controller_name($path_elements) {
    return $path_elements!==null && count($path_elements)>=1
                          && ctype_alnum($path_elements[0]);
}
function path_contains_method_name($path_elements) {
    return $path_elements!==null && count($path_elements)>=2;
}
function get_controller_name($path_elements) {
    global $config;
    
    return (path_contains_controller_name($path_elements))
        ? $path_elements[0] : $config['default_controller'];
}

function get_method_name($path_elements) {
    global $config;

    return (path_contains_method_name($path_elements))
        ? $path_elements[1] : $config['default_method'];
}

function create_controller($controller_name) {
    $controller_classname = ucfirst(strtolower($controller_name));
    $controller_filename = 'controllers/'.$controller_classname.'.php';

    if (!file_exists($controller_filename)) generate_error_404();
    require $controller_filename;

    if (!class_exists($controller_classname)) generate_error_404();

    return new $controller_classname();
}

function call_controller_method($controller, $method_name) {
    $reflectionObject = new ReflectionObject($controller);
    if (!$reflectionObject->hasMethod($method_name)) generate_error_404();
    $reflectionMethod = $reflectionObject->getMethod($method_name);
    $reflectionMethod->invoke($controller);
}

function generate_error_404() {
    header("HTTP/1.0 404 Not Found");
    exit;
}

/* Add the root directory to the include path */
//set_include_path(__DIR__ . PATH_SEPARATOR . $path);

$path_elements = get_path_elements();
$controller_name = get_controller_name($path_elements);
$method_name = get_method_name($path_elements);
$controller = create_controller($controller_name);
call_controller_method($controller, $method_name);
